@@ -42,49 +42,57 @@ class Completion:
     A wrapper for the openai.Completion class
 
     Attributes:
-        collection_name: name of the collection in MongoDB
+        use_case_name: name of the use case so it can be used as a collection name in MongoDB (e.g. "Q&A", "Quiz_Generation")
         config: A Config object
         model: The model to use for the completion
         model_params: A dictionary of parameters used to create the GPT-3 completion
+        experiment_metadata: A dictionary of pre-defined metadata in project_experiment.ini about the experiment
+        project_metadata: A dictionary of pre-defined metadata in project.ini about the project
+        extra_params: Any extra parameters that should be stored in the MongoDB document
     """
 
     def __init__(self,
-                 collection_name: str,
+                 use_case_name: str,
                  config: Config,
                  model: str = "text-davinci-002",
                  model_params: Dict[str, Any] = None,
+                 experiment_metadata: Dict[str, Any] = None,
+                 project_metadata: Dict[str, Any] = None,
                  extra_params: Dict[Any, Any] = None):
 
-        logging.info(f"Initializing Completion object for collection {collection_name}")
+        logging.info(f"Initializing Completion object for collection {use_case_name}")
         logging.info(f"GPT-3 Model: {model}")
 
-        self.collection_name = collection_name
+        self.use_case_name = use_case_name
         self.config = config
         self.mongo_client = config.mongo_client
         self.openai_client = config.openai_client
 
+        self.model_params = {
+            "model": model,
+            "max_tokens": 256,
+            "temperature": 0.7,
+            "top_p": 1,
+            "n": 1,
+            "suffix": None,
+            "stop": None,
+            "best_of": 1,
+            "stream": False,
+            "logprobs": 5,
+            "presence_penalty": 0.0,
+            "frequency_penalty": 0.0,
+            "user": "prosus-AI-team",
+            "echo": False,
+        }
         if model_params is None:
-            self.model_params = {
-                "model": model,
-                "max_tokens": 256,
-                "temperature": 0.7,
-                "top_p": 1,
-                "n": 1,
-                "suffix": None,
-                "stop": None,
-                "best_of": 1,
-                "stream": False,
-                "logprobs": None,
-                "presence_penalty": 0.0,
-                "frequency_penalty": 0.0,
-                "user": "prosus-AI-team",
-                "echo": False,
-            }
             logging.info(f"No model_params provided. Using default params: {self.model_params}")
         else:
-            self.model_params = model_params
+            self.model_params.update(model_params)
             self.model_params["model"] = model
+            logging.info(f"Updated default params. Updated params: {self.model_params}")
 
+        self.experiment_metadata = experiment_metadata
+        self.project_metadata = project_metadata
         self.extra_params = extra_params
 
     def create(self,
@@ -129,14 +137,16 @@ class Completion:
             "completion_text": response["choices"][0]["text"],
             "openai_response": response.__dict__['_previous'],
             "model_params": self.model_params,
-            "created_at": str(datetime.utcnow())
+            "created_at": str(datetime.utcnow()),
+            "experiment_metadata": self.experiment_metadata,
+            "project_metadata": self.project_metadata
         }
 
         if self.extra_params is not None:
             document.update(self.extra_params)
 
         mongo_db = self.mongo_client[self.config.mongo_db_name]
-        mongo_collection = mongo_db[self.collection_name]
+        mongo_collection = mongo_db[self.use_case_name]
         mongo_collection.insert_one(document)
 
         return document
@@ -154,7 +164,7 @@ class Completion:
         """
 
         mongo_db = self.mongo_client[self.config.mongo_db_name]
-        mongo_collection = mongo_db[self.collection_name]
+        mongo_collection = mongo_db[self.use_case_name]
         if prompt_version:
             return list(mongo_collection.find({"prompt": prompt, "prompt_version": prompt_version}))
         else:
@@ -173,7 +183,7 @@ class Completion:
         from bson import ObjectId
 
         mongo_db = self.mongo_client[self.config.mongo_db_name]
-        mongo_collection = mongo_db[self.collection_name]
+        mongo_collection = mongo_db[self.use_case_name]
         return mongo_collection.find_one({"_id": ObjectId(completion_id)})
 
     def get_completions_using_mongodb_filter(self, filters: Dict[Any, Any]) -> List[Dict[Any, Any]]:
@@ -187,5 +197,16 @@ class Completion:
             A dictionary containing the completion
         """
         mongo_db = self.mongo_client[self.config.mongo_db_name]
-        mongo_collection = mongo_db[self.collection_name]
+        mongo_collection = mongo_db[self.use_case_name]
         return list(mongo_collection.find(filters))
+
+    @staticmethod
+    def get_use_cases(config) -> List[str]:
+        """
+        Get all existing use cases (e.g. MongoDB collections)
+
+        Returns:
+            A list of use-cases/collections
+        """
+        mongo_db = config.mongo_client[config.mongo_db_name]
+        return list(mongo_db.list_collection_names())
